@@ -46,5 +46,5 @@ if __name__ == "__main__":
     app = create_app(settings=settings)
 
     uvicorn.run(
-        app, host=os.getenv("HOST", "localhost"), port=int(os.getenv("PORT", 8000))
+        app, host=os.getenv("HOST", settings.host), port=int(os.getenv("PORT", settings.port))
     )
